@@ -1,7 +1,7 @@
 #
 # bio.rb - Loading all BioRuby modules
 #
-#   Copyright (C) 2001-2004 KATAYAMA Toshiaki <k@bioruby.org>
+#   Copyright (C) 2001-2005 KATAYAMA Toshiaki <k@bioruby.org>
 #
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -17,11 +17,11 @@
 #  License along with this library; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 #
-#  $Id: bio.rb,v 1.41 2004/12/13 05:17:48 k Exp $
+#  $Id: bio.rb,v 1.44 2005/08/10 12:56:37 k Exp $
 #
 
 module Bio
-  BIORUBY_VERSION = [0, 6, 2].extend(Comparable)
+  BIORUBY_VERSION = [0, 6, 4].extend(Comparable)
 end
 
 
@@ -107,6 +107,7 @@ require 'bio/io/flatfile'
 require 'bio/io/flatfile/indexer'
 require 'bio/io/flatfile/index'
 require 'bio/io/flatfile/bdb'
+require 'bio/io/fastacmd'
 require 'bio/io/fetch'
 require 'bio/io/sql'
 
@@ -115,6 +116,7 @@ require 'bio/io/keggapi'
 require 'bio/io/pubmed'
 require 'bio/io/das'
 require 'bio/io/ddbjxml'
+require 'bio/io/higet'
 #require 'bio/io/esoap'
 #require 'bio/io/brdb'
 
@@ -130,6 +132,7 @@ require 'bio/appl/blast/wublast'
 require 'bio/appl/blast/rexml'
 require 'bio/appl/blast/xmlparser'
 require 'bio/appl/blast/format8'
+require 'bio/appl/bl2seq/report'
 require 'bio/appl/hmmer'
 require 'bio/appl/hmmer/report'
 require 'bio/appl/emboss'
@@ -147,3 +150,7 @@ require 'bio/appl/sim4/report'
 require 'bio/appl/spidey/report'
 require 'bio/appl/blat/report'
 
+### Utilities
+
+require 'bio/util/sirna'
+
