@@ -17,7 +17,7 @@
 #  License along with this library; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 #
-#  $Id: report.rb,v 1.3 2003/02/26 10:02:38 k Exp $
+#  $Id: report.rb,v 1.6 2005/06/21 07:03:28 ngoto Exp $
 #
 
 require 'bio/appl/blast/xmlparser'
@@ -44,6 +44,9 @@ module Bio
 
     class Report
 
+      # for Bio::FlatFile support (only for XML data)
+      DELIMITER = RS = "</BlastOutput>\n"
+
       def self.xmlparser(data)
 	self.new(data, :xmlparser)
       end
@@ -179,6 +182,7 @@ module Bio
 	def evalue;		@hsps.first.evalue;		end
 	def bit_score;		@hsps.first.bit_score;		end
 	def identity;		@hsps.first.identity;		end
+	def percent_identity;	@hsps.first.percent_identity;	end
 	def overlap;		@hsps.first.align_len;		end
 
 	def query_seq;		@hsps.first.qseq;		end
